/*
 * busywidget.cpp - cool animating widget
 * Copyright (C) 2001, 2002  Justin Karneges
 *                           Hideaki Omuro
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include"busywidget.h"

#include<qtimer.h>
#include<qpainter.h>
#include<math.h>

/////////////////////////////////////////////////////////////////////////////
// common defines
//
#define FPS          20 // frequency of update

#define SPINRATE     24 // 1024ths of a revolution per frame
#define SPINOFFSET    4 // frames each panel is offset by

#define COLORSTOPPED  0xD3D0D3 // color when stopped
#define COLORSPINNING 0xFF00FF // color when spinning
#define COLORSHADOW   0x000000 // color of shadow

/////////////////////////////////////////////////////////////////////////////
// derived defines
//
#define MSECSPERFRAME	(1000 / FPS)

/////////////////////////////////////////////////////////////////////////////
// declared later
//
extern char psigraphic[];

/////////////////////////////////////////////////////////////////////////////
// code
//
BusyWidget::BusyWidget(QWidget *parent, const char *name)
:QWidget(parent, name)
{
	v_isActive = FALSE;
	frame = 0;
	at = 0;

	t = 0;

	pcountdown = 0;
	ocountdown = 0;

	setFixedWidth(82);
	setFixedHeight(19);
	setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));

	int i;
	for(i = 0; i < 5; i++)
		panel[i].SetHeight(17);

	renderPixmap();
}

void BusyWidget::start()
{
	if(v_isActive)
		return;

	v_isActive = TRUE;

	if(!pcountdown)
		pcountdown = SPINOFFSET * 4 + 1;

	if(!t)
	{
		t = new QTimer(this);
		connect(t, SIGNAL(timeout()), SLOT(animate()));
		animate();
		t->start(MSECSPERFRAME);
	}
}

void BusyWidget::stop()
{
	if(!v_isActive)
		return;

	v_isActive = FALSE;

	if(!ocountdown)
		ocountdown = SPINOFFSET * 4 + 1;
}

void BusyWidget::animate()
{
	int i;
	for(i = 0; i < 5; i++)
		panel[i].Spin(SPINRATE);

	if(pcountdown)
		if(!(--pcountdown % SPINOFFSET))
			panel[pcountdown / SPINOFFSET].spinning = true;

	if(ocountdown)
		if(!(--ocountdown % SPINOFFSET))
			panel[ocountdown / SPINOFFSET].spinning = false;

	if(!v_isActive)
	{
		bool isValid = false;
		for(i = 0; i < 5; i++)
			if(panel[i].spinning || panel[i].GetAngle() != 0 || panel[i].alpha != 0)
				isValid = true;

		if(!isValid)
		{
			delete t;
			t = 0;
		}
	}

	renderPixmap();
	render();
}

void BusyWidget::paintEvent(QPaintEvent *)
{
	render();
}

void BusyWidget::resizeEvent(QResizeEvent *)
{
	renderPixmap();
}

void BusyWidget::render()
{
	QPainter p(this);
	p.drawPixmap(0,0, pix);
}

void BusyWidget::renderPixmap()
{
	pix.resize(width(), height());
	pix.fill(QColor("#406080"));

	QPainter p(&pix);

	int i, j, k, l;
	int row;

	for(i = 0; i < 5; i++)
	{
		int o = panel[i].GetModOffset();

		CColor c1(COLORSPINNING), c2(COLORSTOPPED), c3(COLORSHADOW);
		CColor t = c1.Alpha(c2, panel[i].alpha * 8);

		CColor a = t.Alpha(c3, panel[i].GetShade());

		l = panel[i].GetModHeight();

		double radangle = (double) 3.1415926f * (double) panel[i].GetAngle() / (double) 512;
		int step = (int)((double)1024 / cos(radangle));
		step = step < 0 ? -step : step;

		int n = (int)((double)1024 * cos(radangle) * 17 / 2);
		n = n < 0 ? -n : n;

		row = 8192 - step * n / 1024;

		QColor clr(a.GetR(), a.GetG(), a.GetB());

		for(j = 0; j < l; j++)
		{
			int m = row / 1024 + 1;
			for(k = 0; k < 16; k++)
			{
				p.setPen(psigraphic[i * 304 + m * 16 + k] ? Qt::black : clr);
				p.drawPoint(i * 16 + k + 1, o + j + 1);
			}
			row += step;
		}
	}

	p.setPen(Qt::black);
	p.drawRect(0, 0, width(), height());
}

/////////////////////////////////////////////////////////////////////////
// stuff beyond here for animating rotating psi panels
//

// color stuff
CColor CColor::Alpha(CColor clr, int alpha)
{
	int ialpha = 256 - alpha;

	int r, g, b;
	r = (alpha * GetR() + ialpha * clr.GetR()) / 256;
	g = (alpha * GetG() + ialpha * clr.GetG()) / 256;
	b = (alpha * GetB() + ialpha * clr.GetB()) / 256;

	return CColor(r, g, b);
}

// panel stuff
CPanel::CPanel(int _height)
{
	height = _height;
	spinning = false;
	angle = 0;
	alpha = 0;
}

int CPanel::GetModHeight()
{
	int l = GetAngle();
	if(l > 512)
		l = 1024 - l;
	double radangle = (double) 3.1415926f * (double) l / (double) 512;
	int tmp = (int)(cos(radangle)* (double) height);
	return tmp < 0 ? -tmp : tmp;
}

int CPanel::GetShade()
{
	int l = GetAngle() + 128;
	if(GetAngle() >= 256 && GetAngle() < 768)
		l += 512;
	if(l >= 1024)
		l %= 1024;
	if(l == 0)
		l += 1024;
	double radangle = (double) 3.1415926f * (double) l / (double) 512;
	return 128 + (int)(cos(radangle)* (double) 128);
}

void CPanel::Spin(int n)
{
	int i = angle + n;
	if(!spinning)
	{
		if(i >= 1024)
			SetAngle(0);
		if(angle < 512 && i >= 512)
			SetAngle(0);
		if(angle)
			SetAngle(i);
	}
	else
		SetAngle(i);
	if(spinning)
	{
		if(alpha < 32)
			alpha+=2;
	}
	else
	{
		if(alpha)
			alpha-=2;
	}
}

char psigraphic[304*5] =
{
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,

 	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0,
	 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0,
	 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0,
	 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0,
	 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0,
	 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0,
	 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,

	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0,
	 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,

	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,

	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
};
