/*
 * userlist.cpp - high-level roster
 * Copyright (C) 2001, 2002  Justin Karneges
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include"userlist.h"

#include"kmp/types.h"
#include"common.h"

using namespace Jabber;


//----------------------------------------------------------------------------
// UserResource
//----------------------------------------------------------------------------
UserResource::UserResource()
:Resource()
{
}

UserResource::UserResource(const Resource &r)
{
	setResource(r);
}

UserResource::~UserResource()
{
}

void UserResource::setResource(const Resource &r)
{
	setName(r.name());
	setStatus(r.status());
}

const QString & UserResource::versionString() const
{
	return v_ver;
}

void UserResource::setVersionString(const QString &s)
{
	v_ver = s;
}

bool operator<(const UserResource &r1, const UserResource &r2)
{
	return r1.priority() > r2.priority();
}

bool operator<=(const UserResource &r1, const UserResource &r2)
{
	return r1.priority() >= r2.priority();
}

bool operator==(const UserResource &r1, const UserResource &r2)
{
	return r1.priority() == r2.priority();
}

bool operator>(const UserResource &r1, const UserResource &r2)
{
	return r1.priority() < r2.priority();
}

bool operator>=(const UserResource &r1, const UserResource &r2)
{
	return r1.priority() <= r2.priority();
}


//----------------------------------------------------------------------------
// UserResourceList
//----------------------------------------------------------------------------
UserResourceList::UserResourceList()
:QValueList<UserResource>()
{
}

UserResourceList::~UserResourceList()
{
}

UserResourceList::Iterator UserResourceList::find(const QString & _find)
{
	for(UserResourceList::Iterator it = begin(); it != end(); ++it) {
		if((*it).name() == _find)
			return it;
	}

	return end();
}

UserResourceList::Iterator UserResourceList::priority()
{
	UserResourceList::Iterator highest = end();

	for(UserResourceList::Iterator it = begin(); it != end(); ++it) {
		if(highest == end() || (*it).priority() > (*highest).priority())
			highest = it;
	}

	return highest;
}

UserResourceList::ConstIterator UserResourceList::find(const QString & _find) const
{
	for(UserResourceList::ConstIterator it = begin(); it != end(); ++it) {
		if((*it).name() == _find)
			return it;
	}

	return end();
}

UserResourceList::ConstIterator UserResourceList::priority() const
{
	UserResourceList::ConstIterator highest = end();

	for(UserResourceList::ConstIterator it = begin(); it != end(); ++it) {
		if(highest == end() || (*it).priority() > (*highest).priority())
			highest = it;
	}

	return highest;
}

void UserResourceList::sort()
{
	qHeapSort(*this);
}


//----------------------------------------------------------------------------
// UserListItem
//----------------------------------------------------------------------------
UserListItem::UserListItem(bool self)
{
	v_inList = false;
	v_self = self;
}

UserListItem::~UserListItem()
{
}

bool UserListItem::inList() const
{
	return v_inList;
}

void UserListItem::setJid(const Jid &j)
{
	LiveRosterItem::setJid(j);

	int n = jid().full().find('@');
	if(n == -1)
		v_isTransport = true;
	else
		v_isTransport = false;
}

bool UserListItem::isTransport() const
{
	return v_isTransport;
}

bool UserListItem::isAvailable() const
{
	return !v_url.isEmpty();
}

bool UserListItem::isAway() const
{
	int status;
	if(!isAvailable())
		status = STATUS_OFFLINE;
	else
		status = makeSTATUS((*userResourceList().priority()).status());

	if(status == STATUS_AWAY || status == STATUS_XA || status == STATUS_DND)
		return true;
	else
		return false;
}

QDateTime UserListItem::lastAvailable() const
{
	return v_t;
}

const QString & UserListItem::presenceError() const
{
	return v_perr;
}

bool UserListItem::isSelf() const
{
	return v_self;
}

void UserListItem::setInList(bool b)
{
	v_inList = b;
}

void UserListItem::setLastAvailable(const QDateTime &t)
{
	v_t = t;
}

void UserListItem::setPresenceError(const QString &e)
{
	v_perr = e;
}

UserResourceList & UserListItem::userResourceList()
{
	return v_url;
}

UserResourceList::Iterator UserListItem::priority()
{
	return v_url.priority();
}

const UserResourceList & UserListItem::userResourceList() const
{
	return v_url;
}

UserResourceList::ConstIterator UserListItem::priority() const
{
	return v_url.priority();
}

QString UserListItem::makeTip(bool trim) const
{
	QString str = QString("<qt>");
	QString nick = jidnick(jid().full(), name());
	str += QString("<nobr>%1</nobr>").arg(nick);
	if(jid().full() != nick)
		str += QString("<br>[%1]").arg(jid().full());

	// subscription
	if(!v_self)
		str += QString("<br><nobr>") + QString(QObject::tr("Subscription") + ": %1").arg(subscription().toString()) + "</nobr>";

	// resources
	if(!userResourceList().isEmpty()) {
		UserResourceList srl = userResourceList();
		srl.sort();

		for(UserResourceList::ConstIterator rit = srl.begin(); rit != srl.end(); ++rit) {
			const UserResource &r = *rit;
			QString name;
			if(!r.name().isEmpty())
				name = r.name();
			else
				name = QObject::tr("[blank]");

			int status = makeSTATUS(r.status());
			QString istr = "offline";
			if(status == STATUS_ONLINE)
				istr = "online";
			else if(status == STATUS_AWAY)
				istr = "away";
			else if(status == STATUS_XA)
				istr = "xa";
			else if(status == STATUS_DND)
				istr = "dnd";
			str += QString("<br><nobr>") + QString("<img src=\"%1\"> ").arg(istr) + QString("<b>%1</b> ").arg(name) + QString("(%1)").arg(r.priority()) + "</nobr>";

			// last status
			if(r.status().timeStamp().isValid()) {
				QString d = r.status().timeStamp().toString();
				str += QString("<br><nobr>") + QObject::tr("Last Status") + " @ " + d + "</nobr>";
			}

			// gabber music
			if(!r.status().songTitle().isEmpty()) {
				QString s;
				if(trim)
					s = clipStatus(r.status().songTitle(), 200, 1);
				else
					s = r.status().songTitle();
				str += QString("<br><nobr>") + QObject::tr("Listening to") + QString(": %1").arg(s) + "</nobr>";
			}

			// client
			if(!r.versionString().isEmpty())
				str += QString("<br><nobr>") + QObject::tr("Using") + QString(": %1").arg(r.versionString()) + "</nobr>";

			// status message
			QString s = r.status().status();
			if(!s.isEmpty()) {
				QString head = QObject::tr("Status Message");
				if(trim)
					s = plain2rich(clipStatus(s, 200, 12));
				else
					s = linkify(plain2rich(clipStatus(s, 200, 12)));
				str += QString("<br><nobr><u>%1</u></nobr><br>%2").arg(head).arg(s);
			}
		}
	}
	else {
		// last available
		if(!lastAvailable().isNull()) {
			QString d = lastAvailable().toString();
			str += QString("<br><nobr>") + QObject::tr("Last Available") + " @ " + d + "</nobr>";
		}

		// presence error
		if(!v_perr.isEmpty()) {
			str += QString("<br><nobr>") + QObject::tr("Presence Error") + QString(": %1").arg(v_perr) + "</nobr>";
		}

		// status message
		QString s = lastUnavailableStatus().status();
		if(!s.isEmpty()) {
			QString head = QObject::tr("Last Status Message");
			if(trim)
				s = plain2rich(clipStatus(s, 200, 12));
			else
				s = linkify(plain2rich(clipStatus(s, 200, 12)));
			str += QString("<br><nobr><u>%1</u></nobr><br>%2").arg(head).arg(s);
		}
	}

	str += "</qt>";

	return str;
}

QString UserListItem::makeDesc() const
{
	return makeTip(false);
}


//----------------------------------------------------------------------------
// UserList
//----------------------------------------------------------------------------
UserList::UserList()
{
}

UserList::~UserList()
{
}

UserListItem *UserList::find(const Jabber::Jid &j)
{
	UserListIt it(*this);
	for(UserListItem *i; (i = it.current()); ++it) {
		if(i->jid().compare(j))
			return i;
	}
	return 0;
}

